from typing import Dict, Any
import os

from logger import Logger
from services.basicservice import BasicService
from servicestub import ServiceStub
from singleton import Singleton


class GpbSbp(BasicService, metaclass=Singleton):
    log: Logger
    cases: Dict[str, Dict[str, Any]]

    def __init__(self):
        super().__init__(os.path.dirname(os.path.abspath(__file__)))

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса GpbSbp')
        self._loadCases('cases.json')

        servicestub.route('/merchant/qrc-data', method='POST', callback=self.qrcData)
        servicestub.route('/merchant/qrc-status', method='POST', callback=self.qrcStatus)
        servicestub.route('/merchant/payment/v1/cash-activate-qrc/params', method='POST', callback=self.cashActivateQrc)
        servicestub.route('/merchant/transfer/return/prepare', method='POST', callback=self.refundPrepare)
        servicestub.route('/merchant/transfer/return/confirm', method='POST', callback=self.refundConfirm)
        servicestub.route('/operation/info', method='POST', callback=self.operationInfo)
        servicestub.route('/merchant/transfer/rollback', method='POST', callback=self.rollback)
        servicestub.route('/merchant/payment/v1/cash-register-qrc', method='POST', callback=self.registerStaticQr)
        servicestub.route('/merchant/payment/v1/cash-register-qrc/<qrcId>/<paramsId>', method='GET', callback=self.extendedStatus)
        servicestub.route('/merchant/payment/v1/cash-deactivate-qrc/params', method='POST', callback=self.deactivateStaticQr)

    def qrcData(self):
        self.log.info("Регистрация платежной ссылки")
        self._loadCases('cases.json')
        return self._createResponse('/merchant/qrc-data')

    def qrcStatus(self):
        self.log.info("Запрос статуса платежа")
        return self._createResponse('/merchant/qrc-status')

    def cashActivateQrc(self):
        self.log.info("Активация кассовой ссылки")
        self._loadCases('cases.json')
        return self._createResponse('/merchant/payment/v1/cash-activate-qrc/params')

    def refundPrepare(self):
        self.log.info("Создание черновика возврата")
        return self._createResponse('/merchant/transfer/return/prepare')

    def refundConfirm(self):
        self.log.info("Подтверждение возврата")
        return self._createResponse('/merchant/transfer/return/confirm')

    def operationInfo(self):
        self.log.info("Статус возврата")
        return self._createResponse('/operation/info')

    def rollback(self):
        self.log.info("Отмена платежа")
        return self._createResponse('/merchant/transfer/rollback')

    def registerStaticQr(self):
        self.log.info("Регистрация кассовой ссылки")
        return self._createResponse('/merchant/payment/v1/cash-register-qrc')

    def extendedStatus(self, qrcId='', paramsId=''):
        self.log.info(f"Запрос расширенного статуса платежа: qrcId = {qrcId}, paramsId = {paramsId}")
        return self._createResponse(f'/merchant/payment/v1/cash-register-qrc/qrcId/paramsId')

    def deactivateStaticQr(self):
        self.log.info("Деактивация кассовой ссылки")
        return self._createResponse('/merchant/payment/v1/cash-deactivate-qrc/params')


instance = GpbSbp()
